@@ -1,7 +1,6 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/hunchentoot/easy-handlers.lisp,v 1.13 2008/02/13 16:02:17 edi Exp $
 
-;;; Copyright (c) 2004-2009, Dr. Edmund Weitz.  All rights reserved.
+;;; Copyright (c) 2004-2010, Dr. Edmund Weitz.  All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -29,6 +28,13 @@
 
 (in-package :hunchentoot)
 
+(defvar *dispatch-table* (list 'dispatch-easy-handlers)
+  "A global list of dispatch functions.")
+
+(defvar *easy-handler-alist* nil
+  "An alist of \(URI acceptor-names function) lists defined by
+DEFINE-EASY-HANDLER.")
+
 (defun compute-real-name (symbol)
   "Computes the `real' paramater name \(a string) from the Lisp
 symbol SYMBOL.  Used in cases where no parameter name is
@@ -50,7 +56,7 @@ NIL unconditionally."
     (string argument)
     (character (and (= (length argument) 1)
                     (char argument 0)))
-    (integer (ignore-errors (parse-integer argument :junk-allowed t)))
+    (integer (ignore-errors* (parse-integer argument :junk-allowed t)))
     (keyword (as-keyword argument :destructivep nil))
     (boolean t)
     (otherwise (funcall type argument))))
@@ -184,7 +190,7 @@ returns a true value.
 
 ACCEPTOR-NAMES \(which is evaluated) can be a list of symbols which
 means that the handler will be returned by DISPATCH-EASY-HANDLERS in
-acceptor which have one of these names \(see ACCEPTOR-NAME).
+acceptors which have one of these names \(see ACCEPTOR-NAME).
 ACCEPTOR-NAMES can also be the symbol T which means that the handler
 will be returned by DISPATCH-EASY-HANDLERS in every acceptor.
 
@@ -288,8 +294,11 @@ argument is provided."
               `(progn
                  (setq *easy-handler-alist*
                        (delete-if (lambda (list)
-                                    (or (equal ,uri (first list))
-                                        (eq ',name (third list))))
+                                    (and (or (equal ,uri (first list))
+                                             (eq ',name (third list)))
+                                         (or (eq ,acceptor-names t)
+                                             (intersection ,acceptor-names
+                                                           (second list)))))
                                   *easy-handler-alist*))
                  (push (list ,uri ,acceptor-names ',name) *easy-handler-alist*)))))
        (defun ,name (&key ,@(loop for part in lambda-list
@@ -317,3 +326,22 @@ defined with DEFINE-EASY-HANDLER, if there is one."
                          (string= (script-name request) uri))
                         (t (funcall uri request))))
         do (return easy-handler)))
+
+(defclass easy-acceptor (acceptor)
+  ()
+  (:documentation "This is the acceptor of the ``easy'' Hunchentoot framework."))
+
+(defmethod acceptor-dispatch-request ((acceptor easy-acceptor) request)
+  "The easy request dispatcher which selects a request handler
+based on a list of individual request dispatchers all of which can
+either return a handler or neglect by returning NIL."
+  (loop for dispatcher in *dispatch-table*
+     for action = (funcall dispatcher request)
+     when action return (funcall action)
+     finally (call-next-method)))
+
+#-:hunchentoot-no-ssl
+(defclass easy-ssl-acceptor (easy-acceptor ssl-acceptor)
+  ()
+  (:documentation "This is an acceptor that mixes the ``easy''
+  Hunchentoot with SSL connections."))
